package com.gic.store.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.store.dto.StoreBrandDTO;
import com.gic.store.service.StoreBrandApiService;
import com.gic.store.utils.ErrorCode;
import com.gic.store.web.qo.PageQO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * @author zhiwj
 * @date 2019/6/27
 */
@RestController
public class StoreBrandController {

    @Autowired
    private StoreBrandApiService storeBrandApiService;

    @RequestMapping("/save-store-brand")
    public RestResponse saveStoreBrand(StoreBrandDTO storeBrandDTO) {
        Integer enterpriseId = 1111;
        if (StringUtils.isBlank(storeBrandDTO.getStoreBrandCode()) ||
                StringUtils.isBlank(storeBrandDTO.getStoreBrandName()) ||
                StringUtils.isBlank(storeBrandDTO.getStoreBrandCategory()) ||
                storeBrandDTO.getStoreSupport() == null ||
                storeBrandDTO.getGoodsSupport() == null
                ) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), ErrorCode.ERR_5.getMsg());
        }
        if (storeBrandDTO.getStoreSupport() == 0 && storeBrandDTO.getGoodsSupport() == 0) {
            return RestResponse.failure(ErrorCode.ERR_5.getCode(), "门店适用或商品使用至少要选一个");
        }
        storeBrandDTO.setEnterpriseId(enterpriseId);
        ServiceResponse<Integer> serviceResponse = storeBrandApiService.saveOrUpdateStoreBrand(storeBrandDTO);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success();
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/page-store-brand")
    public RestResponse pageStoreBrand(String search, PageQO pageQO) {
        Integer enterpriseId = 1111;
        ServiceResponse<Page<StoreBrandDTO>> serviceResponse = storeBrandApiService.listStoreBrand(enterpriseId, search, pageQO.getCurrentPage(), pageQO.getPageSize());
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-store-brand")
    public RestResponse listStoreBrand(String search, Integer storeSupport, Integer goodsSupport) {
        Integer enterpriseId = 1111;
        ServiceResponse<List<StoreBrandDTO>> serviceResponse = storeBrandApiService.listAllStoreBrand(enterpriseId, search, storeSupport, goodsSupport);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/delete-store-brand-batch")
    public RestResponse deleteByIds(String ids) {
        Integer enterpriseId = 1111;
        ServiceResponse<List<StoreBrandDTO>> serviceResponse = storeBrandApiService.deleteByIds(enterpriseId, ids);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/move-store-brand")
    public RestResponse moveStoreBrand(Integer storeBrandId, Integer moveType) {
        Integer enterpriseId = 1111;
        ServiceResponse serviceResponse = storeBrandApiService.moveStoreBrand(enterpriseId, storeBrandId, moveType);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/set-store-brand-sort")
    public RestResponse setStoreBrandSort(Integer storeBrandId, Integer seq) {
        Integer enterpriseId = 1111;
        ServiceResponse serviceResponse = storeBrandApiService.setStoreBrandSort(enterpriseId, storeBrandId, seq);
        if (serviceResponse.isSuccess()) {
            return RestResponse.success(serviceResponse.getResult());
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }


}
